@@ -22,11 +22,14 @@
 
 #include "config.h"
 
-#include <adwaita.h>
-#include <glib/gi18n.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
+
+#include <adwaita.h>
+
 #include "editor-application-private.h"
+#include "editor-document-private.h"
 #include "editor-info-bar-private.h"
 #include "editor-page-private.h"
 #include "editor-sidebar-model-private.h"
@@ -137,6 +140,23 @@ editor_page_document_modified_changed_cb (EditorPage     *self,
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TITLE]);
 }
 
+static void
+editor_page_document_notify_loading_cb (EditorPage     *self,
+                                        GParamSpec     *pspec,
+                                        EditorDocument *document)
+{
+  gboolean loading;
+
+  g_assert (EDITOR_IS_PAGE (self));
+  g_assert (EDITOR_IS_DOCUMENT (document));
+
+  /* Don't allow focus on the textview while loading as it can
+   * cause GTK to try to keep a11y operations up to date.
+   */
+  loading = _editor_document_get_loading (document);
+  gtk_widget_set_can_focus (GTK_WIDGET (self), !loading);
+}
+
 static void
 editor_page_document_notify_title_cb (EditorPage     *self,
                                       GParamSpec     *pspec,
@@ -180,6 +200,8 @@ editor_page_document_notify_busy_cb (EditorPage     *self,
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CAN_SAVE]);
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SUBTITLE]);
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TITLE]);
+
+  gtk_text_view_set_editable (GTK_TEXT_VIEW (self->view), !busy);
 }
 
 static void
@@ -189,7 +211,8 @@ editor_page_document_cursor_moved_cb (EditorPage     *self,
   g_assert (EDITOR_IS_PAGE (self));
   g_assert (EDITOR_IS_DOCUMENT (document));
 
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_POSITION_LABEL]);
+  if (!_editor_document_get_loading (document))
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_POSITION_LABEL]);
 }
 
 static void
@@ -264,6 +287,11 @@ editor_page_set_document (EditorPage     *self,
       gtk_text_view_set_buffer (GTK_TEXT_VIEW (self->view),
                                 GTK_TEXT_BUFFER (document));
 
+      g_signal_connect_object (document,
+                               "notify::loading",
+                               G_CALLBACK (editor_page_document_notify_loading_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
       g_signal_connect_object (document,
                                "notify::title",
                                G_CALLBACK (editor_page_document_notify_title_cb),
@@ -992,7 +1020,9 @@ editor_page_dup_title (EditorPage *self)
 gchar *
 editor_page_dup_subtitle (EditorPage *self)
 {
+#ifdef G_OS_UNIX
   static char *docportal = NULL;
+#endif
   g_autoptr(GFile) dir = NULL;
   const char *peek;
   GFile *file;
@@ -1000,8 +1030,10 @@ editor_page_dup_subtitle (EditorPage *self)
   g_return_val_if_fail (EDITOR_IS_PAGE (self), NULL);
   g_return_val_if_fail (EDITOR_IS_DOCUMENT (self->document), NULL);
 
+#ifdef G_OS_UNIX
   if (docportal == NULL)
     docportal = g_strdup_printf ("/run/user/%u/doc/", getuid ());
+#endif
 
   file = editor_document_get_file (self->document);
 
@@ -1025,8 +1057,11 @@ editor_page_dup_subtitle (EditorPage *self)
     }
 
   peek = g_file_peek_path (dir);
+
+#ifdef G_OS_UNIX
   if (g_str_has_prefix (peek, docportal))
     return g_strdup (_("Document Portal"));
+#endif
 
   return _editor_path_collapse (peek);
 }
@@ -1072,17 +1107,9 @@ editor_page_save_cb (GObject      *object,
   g_warning ("Failed to save document: %s", error->message);
 
   parent = gtk_widget_get_native (GTK_WIDGET (self));
-  dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-                                   GTK_DIALOG_MODAL | GTK_DIALOG_USE_HEADER_BAR,
-                                   GTK_MESSAGE_WARNING,
-                                   GTK_BUTTONS_CLOSE,
-                                   "%s",
-                                   _("Failed to save document"));
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
-  g_signal_connect (dialog,
-                    "response",
-                    G_CALLBACK (gtk_window_destroy),
-                    NULL);
+  dialog = adw_message_dialog_new (GTK_WINDOW (parent), _("Failed to save document"), NULL);
+  adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "close", _("_Close"));
+  adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dialog), "%s", error->message);
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
@@ -1123,6 +1150,9 @@ editor_page_save_as_cb (EditorPage           *self,
           GtkSourceNewlineType crlf = _editor_file_chooser_get_line_ending (GTK_FILE_CHOOSER (native));
           g_autoptr(GFile) directory = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (native));
 
+          if (directory == NULL)
+            directory = g_file_get_parent (dest);
+
           if (directory != NULL)
             {
               g_autoptr(GSettings) settings = g_settings_new ("org.gnome.TextEditor");
@@ -1194,6 +1224,12 @@ _editor_page_save_as (EditorPage *self,
       gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (native), directory, NULL);
     }
 
+  if (filename == NULL)
+    {
+      g_autofree char *suggestion = _editor_document_suggest_filename (self->document);
+      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (native), suggestion);
+    }
+
   g_signal_connect_object (native,
                            "response",
                            G_CALLBACK (editor_page_save_as_cb),
@@ -1304,6 +1340,9 @@ editor_page_dup_position_label (EditorPage *self)
 
   g_return_val_if_fail (EDITOR_IS_PAGE (self), NULL);
 
+  if (_editor_document_get_loading (self->document))
+    return NULL;
+
   editor_page_get_visual_position (self, &line, &column);
 
   return g_strdup_printf (_("Ln %u, Col %u"), line + 1, column + 1);
@@ -1496,6 +1535,7 @@ editor_page_delete_draft_cb (GObject      *object,
       editor_session_remove_page (session, self);
       _editor_session_remove_draft (session,
                                     _editor_document_get_draft_id (self->document));
+      gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (self->document), FALSE);
       g_task_return_boolean (task, TRUE);
     }
   else if (!reload)
