@@ -1028,6 +1028,7 @@ _editor_session_remove_window (EditorSession *self,
         }
 
       editor_session_save_async (self,
+                                 TRUE,
                                  NULL,
                                  editor_session_save_for_shutdown_cb,
                                  NULL);
@@ -1233,6 +1234,7 @@ editor_session_save_draft_cb (GObject      *object,
 
 void
 editor_session_save_async (EditorSession       *self,
+                           gboolean             shutting_down,
                            GCancellable        *cancellable,
                            GAsyncReadyCallback  callback,
                            gpointer             user_data)
@@ -1252,6 +1254,7 @@ editor_session_save_async (EditorSession       *self,
   g_variant_builder_add_parsed (&builder, "{'version', <%u>}", 1);
   add_draft_state (self, &builder);
   add_window_state (self, &builder);
+  g_variant_builder_add_parsed (&builder, "{'shutdown', <%b>}", !!shutting_down);
   vstate = g_variant_builder_end (&builder);
 
   state = g_slice_new0 (EditorSessionSave);
@@ -1733,6 +1736,8 @@ editor_session_restore_v1 (EditorSession *self,
   GVariantIter iter;
   GVariant *window;
   gboolean had_failure = FALSE;
+  gboolean restore_pages;
+  gboolean did_shutdown = TRUE;
   guint width, height;
 
   g_assert (EDITOR_IS_SESSION (self));
@@ -1749,7 +1754,16 @@ editor_session_restore_v1 (EditorSession *self,
       default_height = height;
     }
 
-  if (!self->restore_pages ||
+  /* Check if the application crashed. If shutdown==false, then we auto-saved
+   * session state but did not shutdown gracefully. We always restore pages in
+   * that case to avoid data loss of drafts. (Technically they're there, but the
+   * user wouldn't be able to find them unless session-restore is active).
+   */
+  restore_pages = self->restore_pages;
+  if (g_variant_lookup (state, "shutdown", "b", &did_shutdown) && !did_shutdown)
+    restore_pages = TRUE;
+
+  if (!restore_pages ||
       !(windows = g_variant_lookup_value (state, "windows", G_VARIANT_TYPE ("aa{sv}"))) ||
       g_variant_n_children (windows) == 0)
     goto failure;
@@ -1818,7 +1832,7 @@ editor_session_restore_v1 (EditorSession *self,
   return;
 
 failure:
-  if (!self->restore_pages)
+  if (!restore_pages)
     g_debug ("Failed to restore session or nothing to restore");
 
   editor_session_create_window (self);
@@ -2224,7 +2238,7 @@ editor_session_auto_save_timeout_cb (gpointer user_data)
   self->auto_save_source = 0;
 
   g_debug ("Performing auto-save of session state");
-  editor_session_save_async (self, NULL, NULL, NULL);
+  editor_session_save_async (self, FALSE, NULL, NULL, NULL);
 
   return G_SOURCE_REMOVE;
 }
@@ -2455,3 +2469,74 @@ _editor_session_clear_history (EditorSession *self)
         _editor_sidebar_model_remove_draft (self->recents, draft_id);
     }
 }
+
+static void
+editor_session_load_stream_cb (GObject      *object,
+                               GAsyncResult *result,
+                               gpointer      user_data)
+{
+  GtkSourceFileLoader *loader = (GtkSourceFileLoader *)object;
+  g_autoptr(EditorDocument) document = user_data;
+  g_autoptr(GError) error = NULL;
+
+  g_assert (GTK_SOURCE_IS_FILE_LOADER (loader));
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (EDITOR_IS_DOCUMENT (document));
+
+  if (!gtk_source_file_loader_load_finish (loader, result, &error))
+    g_warning ("Failed to read input stream: %s", error->message);
+
+  gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (document), TRUE);
+}
+
+static void
+load_stream_into_document (GInputStream   *stream,
+                           EditorDocument *document)
+{
+  g_autoptr(GtkSourceFileLoader) loader = NULL;
+  g_autoptr(GtkSourceFile) file = NULL;
+
+  g_assert (G_IS_INPUT_STREAM (stream));
+  g_assert (EDITOR_IS_DOCUMENT (document));
+
+  file = gtk_source_file_new ();
+  loader = gtk_source_file_loader_new_from_stream (GTK_SOURCE_BUFFER (document), file, stream);
+  gtk_source_file_loader_load_async (loader,
+                                     G_PRIORITY_DEFAULT,
+                                     NULL,
+                                     NULL, NULL, NULL,
+                                     editor_session_load_stream_cb,
+                                     g_object_ref (document));
+}
+
+void
+editor_session_open_stream (EditorSession *session,
+                            EditorWindow  *window,
+                            GInputStream  *stream)
+{
+  g_autoptr(EditorDocument) new_document = NULL;
+  EditorPage *old_page;
+  EditorPage *new_page;
+
+  g_return_if_fail (EDITOR_IS_SESSION (session));
+  g_return_if_fail (!window || EDITOR_IS_WINDOW (window));
+  g_return_if_fail (G_IS_INPUT_STREAM (stream));
+
+  if (window == NULL)
+    window = find_or_create_window (session);
+
+  /* If the window has a single empty page within it, just close that
+   * page and let our new page replace it.
+   */
+  if (editor_window_get_n_pages (window) == 1 &&
+      (old_page = editor_window_get_nth_page (window, 0)) &&
+      editor_page_get_can_discard (old_page))
+    _editor_window_remove_page (window, old_page);
+
+  new_document = editor_document_new_draft ();
+  new_page = editor_session_add_document (session, window, new_document);
+
+  load_stream_into_document (stream, new_document);
+  _editor_page_raise (new_page);
+  _editor_session_mark_dirty (session);
+}
