@@ -503,6 +503,9 @@ editor_window_constructed (GObject *object)
                           NULL);
   gtk_widget_add_css_class (zoom_in, "circular");
   gtk_widget_add_css_class (zoom_in, "flat");
+  gtk_accessible_update_property (GTK_ACCESSIBLE (zoom_in),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL,
+                                  _("Zoom in"), -1);
   zoom_out = g_object_new (GTK_TYPE_BUTTON,
                            "action-name", "page.zoom-out",
                           "child", g_object_new (GTK_TYPE_IMAGE,
@@ -512,6 +515,9 @@ editor_window_constructed (GObject *object)
                            NULL);
   gtk_widget_add_css_class (zoom_out, "circular");
   gtk_widget_add_css_class (zoom_out, "flat");
+  gtk_accessible_update_property (GTK_ACCESSIBLE (zoom_out),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL,
+                                  _("Zoom out"), -1);
   self->zoom_label = g_object_new (GTK_TYPE_LABEL,
                                    "hexpand", TRUE,
                                    "label", "100%",
@@ -753,6 +759,36 @@ on_show_help_overlay_cb (GtkWidget  *widget,
     }
 }
 
+static gboolean
+title_query_tooltip_cb (EditorWindow *self,
+                        int           x,
+                        int           y,
+                        gboolean      keyboard,
+                        GtkTooltip   *tooltip)
+{
+  g_autofree char *text = NULL;
+  EditorDocument *document;
+  EditorPage *page;
+  GFile *file;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+  g_assert (GTK_IS_TOOLTIP (tooltip));
+
+  if (!(page = editor_window_get_visible_page (self)) ||
+      !(document = editor_page_get_document (page)) ||
+      !(file = editor_document_get_file (document)))
+    return FALSE;
+
+  if (g_file_is_native (file))
+    text = g_file_get_path (file);
+  else
+    text = g_file_get_uri (file);
+
+  gtk_tooltip_set_text (tooltip, text);
+
+  return TRUE;
+}
+
 static gboolean
 indicator_to_boolean (GBinding     *binding,
                       const GValue *from_value,
@@ -778,6 +814,13 @@ editor_window_dispose (GObject *object)
   editor_signal_group_set_target (self->page_signals, NULL);
   editor_signal_group_set_target (self->document_signals, NULL);
 
+  if (self->inhibit_cookie != 0)
+    {
+      gtk_application_uninhibit (GTK_APPLICATION (EDITOR_APPLICATION_DEFAULT),
+                                 self->inhibit_cookie);
+      self->inhibit_cookie = 0;
+    }
+
   G_OBJECT_CLASS (editor_window_parent_class)->dispose (object);
 }
 
@@ -881,6 +924,7 @@ editor_window_class_init (EditorWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_tab_view_close_page_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_tab_view_setup_menu_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_tab_view_create_window_cb);
+  gtk_widget_class_bind_template_callback (widget_class, title_query_tooltip_cb);
 
   gtk_widget_class_install_action (widget_class, "win.alternate-help-overlay", NULL, on_show_help_overlay_cb);
   gtk_widget_class_install_action (widget_class, "win.undo-close-page", NULL, on_undo_close_page_cb);
@@ -1089,6 +1133,58 @@ modified_to_icon (GBinding     *binding,
   return TRUE;
 }
 
+static void
+editor_window_update_inhibit (EditorWindow *self)
+{
+  gboolean inhibit = FALSE;
+  guint n_pages;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+
+  n_pages = editor_window_get_n_pages (self);
+
+  for (guint i = 0; i < n_pages; i++)
+    {
+      EditorPage *page = editor_window_get_nth_page (self, i);
+
+      if (editor_page_get_is_modified (page))
+        {
+          inhibit = TRUE;
+          break;
+        }
+    }
+
+  if ((inhibit && self->inhibit_cookie != 0) ||
+      (!inhibit && self->inhibit_cookie == 0))
+    return;
+
+  if (inhibit)
+    {
+      self->inhibit_cookie =
+        gtk_application_inhibit (GTK_APPLICATION (EDITOR_APPLICATION_DEFAULT),
+                                 GTK_WINDOW (self),
+                                 GTK_APPLICATION_INHIBIT_LOGOUT,
+                                 _("There are unsaved documents"));
+    }
+  else
+    {
+      gtk_application_uninhibit (GTK_APPLICATION (EDITOR_APPLICATION_DEFAULT),
+                                 self->inhibit_cookie);
+      self->inhibit_cookie = 0;
+    }
+}
+
+static void
+page_notify_is_modified_cb (EditorWindow *self,
+                            GParamSpec   *pspec,
+                            EditorPage   *page)
+{
+  g_assert (EDITOR_IS_WINDOW (self));
+  g_assert (EDITOR_IS_PAGE (page));
+
+  editor_window_update_inhibit (self);
+}
+
 void
 _editor_window_add_page (EditorWindow *self,
                          EditorPage   *page)
@@ -1109,7 +1205,15 @@ _editor_window_add_page (EditorWindow *self,
                                modified_to_icon, NULL,
                                NULL, NULL);
 
+  g_signal_connect_object (page,
+                           "notify::is-modified",
+                           G_CALLBACK (page_notify_is_modified_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   adw_tab_view_set_selected_page (self->tab_view, tab_page);
+
+  editor_window_update_inhibit (self);
 }
 
 void
@@ -1121,6 +1225,10 @@ _editor_window_remove_page (EditorWindow *self,
   g_return_if_fail (EDITOR_IS_WINDOW (self));
   g_return_if_fail (EDITOR_IS_PAGE (page));
 
+  g_signal_handlers_disconnect_by_func (page,
+                                        G_CALLBACK (page_notify_is_modified_cb),
+                                        self);
+
   tab_page = adw_tab_view_get_page (self->tab_view, GTK_WIDGET (page));
   adw_tab_view_close_page (self->tab_view, tab_page);
 
@@ -1131,6 +1239,11 @@ _editor_window_remove_page (EditorWindow *self,
       if (self->visible_page != NULL)
         editor_page_grab_focus (self->visible_page);
     }
+
+  if (self->visible_page == NULL)
+    gtk_window_set_title (GTK_WINDOW (self), _("Text Editor"));
+
+  editor_window_update_inhibit (self);
 }
 
 /**
